import { mudMessage, MUDMessagePriority } from "../utility/mud-messenger";
export class MUDStatDefinitionAccessors {
    constructor() {
        this._statDefinitions = [];
    }
    registerStatDefinition(statDefinition) {
        const alreadyRegistered = this._statDefinitions.some(statDef => statDef.statID === statDefinition.statID);
        if (alreadyRegistered) {
            mudMessage(MUDMessagePriority.warning, `Stat Definition with ID ${statDefinition.statID} already registered`);
        }
        else {
            this._statDefinitions.push(statDefinition);
        }
    }
    getStatDefinitions() {
        return this._statDefinitions;
    }
    getStatDefinitionByID(statDefinitionID) {
        return this._statDefinitions.find(def => def.statID === statDefinitionID);
    }
    editStatDefinitionByID(statDefinitionID, edits) {
        const stat = this.getStatDefinitionByID(statDefinitionID);
        console.log(stat);
        if (stat) {
            stat.displayName = edits.displayName;
            stat.minValue = edits.minValue;
            stat.maxValue = edits.maxValue;
            console.log(stat);
            return stat;
        }
        else {
            mudMessage(MUDMessagePriority.warning, `No stat definition found that matches ID ${statDefinitionID}, cannot edit.`);
            return null;
        }
    }
    deleteStatDefinitionByID(statDefinitionID) {
        const stat = this.getStatDefinitionByID(statDefinitionID);
        const index = this._statDefinitions.indexOf(stat);
        if (index > -1) {
            this._statDefinitions.splice(index, 1);
        }
        else {
            mudMessage(MUDMessagePriority.warning, `No stat definition found that matches ID ${statDefinitionID}, cannot delete`);
            return null;
        }
    }
}
//# sourceMappingURL=stat-definition.acc.js.map