var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var MUDGame_1;
import { MUDLevel } from './level';
import { mudMessage, MUDMessagePriority } from '../utility/mud-messenger';
import { MUDPlayer } from './player';
import { MUDStatDefinitionAccessors } from "../accessors/stat-definition.acc";
import { staticImplements } from "../utility/class-decorators";
import { MUDStatDefinition } from "./stat-definition";
import { MUDPlayerAccessors } from "../accessors/player.acc";
let MUDGame = MUDGame_1 = class MUDGame {
    constructor() {
        // TODO:  Move these into separate "managers"
        // TODO:  Observables?
        this.StatDefinitions = new MUDStatDefinitionAccessors();
        this.Players = new MUDPlayerAccessors();
        // private _ResetObjects() {
        //     this.StatDefinitions = new MUDStatDefinitionAccessors();
        //     this.Players = new MUDPlayerAccessors();
        // }
        // Level Accessors
        this._levels = [];
    }
    get levelCount() {
        return this._levels.length;
    }
    createLevel(name, width, height) {
        const level = new MUDLevel(name, width, height);
        this._levels.push(level);
        return level;
    }
    getLevel(id) {
        if (typeof id === 'number') {
            if (this._levels[id]) {
                return this._levels[id];
            }
            else {
                mudMessage(MUDMessagePriority.error, `Level not found with index ${id}`);
                return null;
            }
        }
        if (typeof id === 'string') {
            const levelMatch = this._levels.find(level => level.info.levelID === id);
            if (levelMatch) {
                return levelMatch;
            }
            else {
                mudMessage(MUDMessagePriority.error, `Level  not found with identifier ${id}`);
                return null;
            }
        }
        mudMessage(MUDMessagePriority.error, 'No valid room identifier provided');
        return null;
    }
    static Serialize(game) {
        return {
            _players: game.Players.getPlayers().map((player) => MUDPlayer.Serialize(player)),
            _statDefinitions: game.StatDefinitions.getStatDefinitions().map((statDef) => MUDStatDefinition.Serialize(statDef))
        };
    }
    static Deserialize(serializedGame) {
        const game = new MUDGame_1();
        // Set Stat Definitions
        game.StatDefinitions['_statDefinitions'] = serializedGame._statDefinitions.map((s_statDef) => {
            return MUDStatDefinition.Deserialize(s_statDef);
        });
        // SetPlayers
        game.Players['_players'] = serializedGame._players.map((sPlayer) => MUDPlayer.Deserialize(sPlayer));
        return game;
    }
};
MUDGame = MUDGame_1 = __decorate([
    staticImplements()
], MUDGame);
export { MUDGame };
//# sourceMappingURL=game.js.map