var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var MUDStat_1;
import { clamp } from 'lodash';
import { MUDStatMod, MUDStatModPhase, MUDStatModType } from './stat-mod';
import { staticImplements } from '../utility/class-decorators';
import { MUD } from "../mudjs";
let MUDStat = MUDStat_1 = class MUDStat {
    constructor(statDefinition, baseValue) {
        this._mods = [];
        this._statDefinition = statDefinition;
        this._baseValue = baseValue;
    }
    get displayName() {
        return this._statDefinition.displayName;
    }
    get baseValue() {
        return this._baseValue;
    }
    get statID() {
        return this._statDefinition.statID;
    }
    get totalValue() {
        const earlyModResult = this.calcMods(MUDStatModPhase.early, this._baseValue);
        const normalModResult = this.calcMods(MUDStatModPhase.normal, earlyModResult);
        const lateModResult = this.calcMods(MUDStatModPhase.late, normalModResult);
        return clamp(lateModResult, this._statDefinition.minValue, this._statDefinition.maxValue);
    }
    AddToBaseValue(valueToAdd) {
        return (this._baseValue += valueToAdd);
    }
    AddModifier(value, type, phase = MUDStatModPhase.normal) {
        this._mods.push(new MUDStatMod(value, type, phase));
    }
    calcMods(phase, value) {
        const mods = this._mods.filter(mod => mod.phase === phase);
        return mods.reduce((total, current) => {
            switch (current.type) {
                case MUDStatModType.add:
                    return total + current.value;
                case MUDStatModType.subtract:
                    return total - current.value;
                case MUDStatModType.multiply:
                    return total * current.value;
                case MUDStatModType.divide:
                    return total / current.value;
            }
        }, value);
    }
    static Serialize(stat) {
        return {
            _statDefinitionID: stat._statDefinition.statID,
            _baseValue: stat._baseValue,
            _mods: stat._mods.map(mod => MUDStatMod.Serialize(mod))
        };
    }
    static Deserialize(serializedStat) {
        const statDefinition = MUD.MUDGame.StatDefinitions.getStatDefinitionByID(serializedStat._statDefinitionID);
        const stat = new MUDStat_1(statDefinition, serializedStat._baseValue);
        stat._mods = serializedStat._mods.map(mod => {
            return MUDStatMod.Deserialize(mod);
        });
        return stat;
    }
};
MUDStat = MUDStat_1 = __decorate([
    staticImplements()
], MUDStat);
export { MUDStat };
//# sourceMappingURL=stat.js.map